#include "lcd/Nokia6100Driver.h"
#include "lcd/Widgets.h"


rom const BYTE NUMBER_OFFSETS_X[] = {2,2,3,2,3,2,3,2,3,2};

rom const BYTE LED_PLACEMENT_X[] = {20,50,80,110,35,65,95};
rom const BYTE LED_PLACEMENT_Y[] = {40,40,40,40,14,14,14};


BarGraphWidget redBar;
BarGraphWidget blueBar;
BarGraphWidget greenBar;

SignalWidget p2pSignal;


PowerIndicatorWidget pwrIndicator;

LEDGroupWidget LEDGroup;


								
#define ARROW_WIDTH 16
#define ARROW_HEIGHT 15


//Does initial drawing of the bargraph widget
void BarGraph_Draw(BarGraphWidget * widget)
{
	float heightcalc;
	BYTE range = widget->max - widget->min;
	BYTE overValue = widget->value - widget->min;
	BYTE BarHeight;
	BYTE t;
	BYTE p;
	BYTE OuterColor;
	
	BYTE right = widget->left + (widget->width - (widget->BorderWeight - 1));
	BYTE bottom = widget->top + (widget->height - (widget->BorderWeight - 1));
	
	
	
	if (widget->Selected)
	{
		OuterColor = widget->OuterBorderSelectedColor;
	}	
	else
	{
		OuterColor = widget->OuterBorderColor;
	}	
	
	heightcalc = ((float) overValue) / ((float)range);
	heightcalc = heightcalc * ((float) (widget->height - (widget->BorderWeight *2)));
	BarHeight = (int) heightcalc;
	
	//Clear the area
	//First draw the outline.
	drawFilledBox(widget->BackColor, widget->left, widget->top, right , bottom);
	
	//First draw the outline.
	drawBox(widget->BorderColor, widget->left, widget->top, right , bottom , widget->BorderWeight);
	
	drawBox(OuterColor, widget->left - widget->OuterBorderWeight, 
	widget->top - widget->OuterBorderWeight, right + widget->BorderWeight  , 
	bottom + widget->BorderWeight, widget->OuterBorderWeight);
	
	//Next draw fill portion
	drawFilledBox(widget->FillColor, widget->left+widget->BorderWeight, (bottom-1) - BarHeight
	, right-1, bottom-1);
	
	widget->LastValue = widget->value;
	widget->LastBarHeight = BarHeight;
	

	
}	
void BarGraph_UpdateBar(BarGraphWidget * widget)
{
	int heightcalc;
	BYTE range = widget->max - widget->min;
	BYTE overValue = widget->value - widget->min;
	BYTE BarHeight;
	BYTE t;
	BYTE p;
	BYTE OuterColor;
	
	BYTE right = widget->left + (widget->width - (widget->BorderWeight - 1));
	BYTE bottom = widget->top + (widget->height - (widget->BorderWeight - 1));
	
	if (widget->value != widget->LastValue)
	{
		if (widget->value > widget->max) widget->value = widget->max;
		if (widget->value < widget->min) widget->value = widget->min;
		heightcalc = (((int) overValue) * 100) / ((int)range);
		heightcalc = heightcalc * (widget->height - (widget->BorderWeight *2));
		BarHeight = heightcalc / 100;
		
		if (BarHeight > widget->LastBarHeight)
		{
			drawFilledBox(widget->FillColor, widget->left+widget->BorderWeight, (bottom-1) - BarHeight
			, right-1, bottom- 1 - widget->LastBarHeight);
		}
		if (BarHeight < widget->LastBarHeight)
		{
			drawFilledBox(widget->BackColor , widget->left+widget->BorderWeight,(bottom-1) - BarHeight 
			, right-1, bottom- 1 - widget->LastBarHeight);
		}
		widget->LastValue = widget->value;
		widget->LastBarHeight = BarHeight;
	}
	if (widget->Selected)
	{
		OuterColor = widget->OuterBorderSelectedColor;
	}	
	else
	{
		OuterColor = widget->OuterBorderColor;
	}

	drawBox(OuterColor, widget->left - widget->OuterBorderWeight, 
	widget->top - widget->OuterBorderWeight, right + widget->BorderWeight  , 
	bottom + widget->BorderWeight, widget->OuterBorderWeight);	
	

	
}	
void BarGraph_SetDefault(BarGraphWidget * widget)
{
	widget->BackColor = BLACK;
	widget->BorderColor = GREEN;
	widget->FillColor = GREEN;
	widget->height=65;
	widget->width=20;
	widget->left=10;
	widget->top=60;
	widget->min=0;
	widget->max=255;
	widget->value = 0;
	widget->LastValue=0;
	widget->BorderWeight=1;
	widget->OuterBorderColor=BLACK;
	widget->OuterBorderSelectedColor=WHITE;
	widget->OuterBorderWeight=2;
	widget->Selected = 1;
	widget->ArrowColor = WHITE;
}

void Signal_SetDefault(SignalWidget * widget)
{
	widget->color = GREEN;
	widget->RSSIv = 1;
	widget->LQQIv = 1;
	widget->visible = 0;
	widget->top = 3;
	widget->left = 112;
	widget->BackColor = BLACK;

}	
void Signal_Update(SignalWidget * widget)
{
	BYTE bars;
	BYTE i;
	
	unsigned int avgSignal = (widget->RSSIv + widget->LQQIv) / 2;
	bars = 1;
	if (avgSignal > 25) bars = 2;
	if (avgSignal > 50) bars = 3;
	if (avgSignal > 80) bars = 4;
	if (avgSignal > 120) bars = 5;

	
	if (widget->visible)
	{
		for (i=0; i < 5; i++)
		{
			BYTE BarColor;
			if (i < bars)
			{
				BarColor = widget->color;
			}	
			else
			{
				BarColor = widget->BackColor;
			}	
			
			drawFilledBox(BarColor, widget->left + (3*i), widget->top + (16-(3*i)-2), widget->left + (3*i)+1, widget->top + 16);  
		}	
	}
	else
	{
		drawFilledBox(widget->BackColor, widget->left, widget->top, widget->left + 16, widget->top + 16);  
	}	
}	


void LED_SetDefault(LEDWidget * widget)
{
	widget->Selected = 1;
	widget->LEDNumber = 1;
	widget->BackColor = BLACK;
	widget->SelectionColor = WHITE;
	widget->FillColor = YELLOW;
	widget->BorderColor = BLACK;
	widget->TextColor = BLACK;
	widget->address = 10;
	widget->VerticalCenter = 40;
	widget->HorizontalCenter = 20;
	widget->visible = 0;
	widget->exists = 0;
	widget->red = 0;
	widget->green = 0;
	widget->blue = 0;
}	
void LED_Update(LEDWidget * widget)
{
	char LEDNumberString[2];
	BYTE OuterCircleColor;
	if (widget->visible)
	{
		if (widget->Selected)
		{
			OuterCircleColor = widget->SelectionColor;
		}
		else
		{
			OuterCircleColor = widget->BorderColor;	
		}		
		
		LEDNumberString[0] = widget->LEDNumber + 48;
		LEDNumberString[1] = 0;
		
		
		
		drawFilledCircle(OuterCircleColor, widget->HorizontalCenter, widget->VerticalCenter, 11);
		drawFilledCircle(widget->FillColor, widget->HorizontalCenter, widget->VerticalCenter, 8);
		LCDString(widget->TextColor, widget->FillColor, LEDNumberString, widget->HorizontalCenter - NUMBER_OFFSETS_X[widget->LEDNumber], 
		widget->VerticalCenter-4);
	}	
	else
	{
		drawFilledCircle(widget->BackColor, widget->HorizontalCenter, widget->VerticalCenter, 12);	
	}	
}


void PowerIndicator_SetDefault(PowerIndicatorWidget * widget)
{
	 widget->visible = 1;
	 widget->state = 1;
	 widget->TextColor = WHITE;
	 widget->BackColor = BLACK;
	 widget->BorderColor = WHITE;
	 widget->FillColorOn = DARKGREEN;
	 widget->FillColorOff = GREY;
	 widget->top = 5;
	 widget->left = 4;
	 widget->BorderWeight = 2;
}

void PowerIndicator_Update(PowerIndicatorWidget * widget)
{
	BYTE FillColor;
	char StateString[2];
	BYTE TextOffset;
	if (widget->state)
	{
		StateString[0] = '1';
		StateString[1] = 0;
	 	FillColor = widget->FillColorOn;
	 	TextOffset = 5;
	} else
	{
		StateString[0] = '0';
		StateString[1] = 0;
		FillColor = widget->FillColorOff;
		TextOffset = 5;
	}		
	if (widget->visible)
	{
		drawFilledBox(FillColor, widget->left, widget->top, widget->left + 15, widget->top + 15); 
		
		LCDString(widget->TextColor, FillColor, StateString, 
		widget->left + TextOffset, widget->top+3);
		
		
		drawBox(widget->BorderColor, widget->left , 
		widget->top, widget->left + 16 - widget->BorderWeight, 
		widget->top + 16 - widget->BorderWeight, widget->BorderWeight);	 
		

	}
	else
	{
		drawFilledBox(widget->BackColor, widget->left, widget->top, widget->left + 16, widget->top + 16); 	
	}	
}	

void LEDGroup_SetDefault(LEDGroupWidget * widget)
{
	BYTE i;
	for (i = 0; i < MAX_LEDS; i++)
	{
		LED_SetDefault(&(widget->LEDs[i]));
		widget->LEDs[i].LEDNumber = i + 1;
		widget->LEDs[i].VerticalCenter = LED_PLACEMENT_Y[i];
		widget->LEDs[i].HorizontalCenter = LED_PLACEMENT_X[i];
	}	
}	
void LEDGroup_Update(LEDGroupWidget * widget)
{
	BYTE i;
	for (i = 0; i < MAX_LEDS; i++)
	{
		LED_Update(&(widget->LEDs[i]));
	}
}	

void LEDGroup_SelectAll(LEDGroupWidget * widget, BYTE state)
{
	BYTE i;
	for (i = 0; i < MAX_LEDS; i++)
	{
		if (widget->LEDs[i].visible)
		{
			widget->LEDs[i].Selected = state;
			LED_Update(&(widget->LEDs[i]));
		}
	}	
}
