#include "HardwareProfile.h"
#include "ConfigApp.h"
#include "mTouchCap_PIC18_CTMU_Physical.h"
#include "mTouchCap_CtmuAPI.h"
#include "WirelessProtocols/MCHP_API.h"
#include "lcd/Nokia6100Driver.h"
#include "lcd/Widgets.h"
#include <delays.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "sound.h"
#include "mTouchCap_2Ch_Slider.h"

#include	"mTouchCap_Timers.h"
#include "Config.h"


#pragma config FOSC = INTIO67, PLLCFG = ON, PWRTEN = OFF, BOREN = OFF, WDTEN = OFF, PBADEN = OFF, LVP = OFF, HFOFST = OFF
#if defined(__EXTENDED18__)
	#pragma config XINST=ON
#else
	#pragma config XINST=OFF
#endif

#define POWER_COMMAND 1
#define SET_RED_COMMAND 2
#define SET_GREEN_COMMAND 3
#define SET_BLUE_COMMAND 4
#define SET_ALL_COMMAND 5
#define CLOSE_COMMAND 7
#define REQUEST_LED_TABLE_COMMAND 10
#define RESET_COMMAND 13
#define REQUEST_POWER_COMMAND 16
#define STORE_LED_COMMAND 21
#define DELETE_LED_COMMAND 22

#define NUM_ELEM(x) (sizeof (x) / sizeof (*(x)))

typedef struct _LEDCommand {
	BYTE commandcode;
	BYTE param1;
	BYTE param2;
} LEDCommand;

void InitializeHardware(void);
void MiWiTimerTick(void);
void LowISR(void);
void CycleRGBArrow(BYTE direction);
void FindP2PConnection(void);
void RequestLEDTable(void);
void setupCapTouch(void);
void RGBValueChange(BYTE value);
void ProcessLEDCommand(LEDCommand * ledcommand);
void GoToSleep(void);
void setLEDState(BYTE LEDIndex, BYTE ColorCode, BYTE val);



//Key touch event handlers
void RedKeyTouchEvent();
void GreenKeyTouchEvent();
void BlueKeyTouchEvent();
void SelectLEDKeyTouchEvent();
void SelectAllKeyTouchEvent();
void ResetRGBKeyTouchEvent();
void MaintenenceKeyTouchEvent();
void PowerKeyTouchEvent();


//this byte is here to allow disabled of some 
//outputs in headers files.
BYTE DisableByte;

BYTE ActivePANIndex;
BYTE FailCount = 0;
#define MAX_FAILURES 3

BYTE GotLEDTable = 0;
long LEDTableRequestStamp = 0;
#define LED_TABLE_TIMEOUT 5000

#define IDLE_TIMEOUT 90000


BYTE MaintenenceMode = 0;



SoundRequest SoundQue[SOUND_QUE_LENGTH];
SoundRequest * SoundRequestPtr = &SoundQue[0];
SoundRequest * SoundServicePtr = &SoundQue[0];





DirectKey RedKey;
DirectKey GreenKey;
DirectKey BlueKey;
DirectKey SelectLEDKey;
DirectKey SelectAllKey;
DirectKey ResetRGBKey;
DirectKey MaintenenceKey;
DirectKey PowerKey;

DirectKey * KeyArray[] = {&RedKey, &GreenKey, &BlueKey, &SelectLEDKey, &SelectAllKey, &ResetRGBKey, &MaintenenceKey, &PowerKey};

Slider2Ch RGBSlider;

#define DEBOUNCE_MS 250



extern WORD channelSelect; 




extern BarGraphWidget redBar;
extern BarGraphWidget blueBar;
extern BarGraphWidget greenBar;



extern SignalWidget p2pSignal;
extern PowerIndicatorWidget pwrIndicator;
extern LEDGroupWidget LEDGroup;






unsigned long SystemTick = 0;
unsigned long SoundStartTick = 0;
unsigned long PowerButtonDownStamp = 0;
unsigned long ActivityStamp = 0;
unsigned long ScanStartStamp = 0;


BYTE SoundPause = 0;
BYTE RotatingDots = 0;


/*This is the low interupt vector. The MRF24J40 configures Timer0
  to interupt about every 4ms. */
  
#pragma interruptlow LowISR
void LowISR(void)
{
		
    if(INTCONbits.TMR0IF)
    {
		// Increment internal tick counter
		MiWiTimerTick();

		// Reset interrupt flag
        INTCONbits.TMR0IF = 0;
        
        SystemTick+=4;
        
        //Service sound
        if (!SoundServicePtr->WasServiced)
        {
	    	if (T3CONbits.TMR3ON || SoundPause) //are we already playing it
	    	{
		    	if ((SystemTick - SoundStartTick) >= SoundServicePtr->duration)
		    	{
			    	SoundOff();
			    	SoundServicePtr->WasServiced = 1;
			    	IncrementServicePtr();
			    }	
		    }	
		    else //new request
		    {
			    if (SoundServicePtr->frequency == -1)
			    {
			    	SoundPause = 1;
			    }
			    else
			    {
					SetSpkFreq(SoundServicePtr->frequency);
					SoundOn();   
				}
			} 
	    }
	    
	    if (RotatingDots)
	    {
		    static long LastRotate = 0;
		    static char NoDots[] = "   ";
		    static char OneDots[] = ".  ";
		    static char TwoDots[] = ".. ";
		    static char *DotArray[3] = {NoDots,OneDots,TwoDots};
		    static BYTE CurrentDotIdx = 0;
		    if ((SystemTick - LastRotate) > 350)
		    {
				LCDString(WHITE, BLACK, DotArray[CurrentDotIdx], 104,30);
				CurrentDotIdx++;
				if (CurrentDotIdx >= 3) CurrentDotIdx = 0;
				LastRotate = SystemTick;
			} 
		} 
		if (MaintenenceMode)
        { 
			if ((SystemTick % 500) == 0) //300ms intervals
			{
				//Request current power and voltage stats from the controller
				LEDCommand ledcommand;
				ledcommand.commandcode = REQUEST_POWER_COMMAND;
				ledcommand.param1 = 255; //dummy data
				ledcommand.param2 = 255; //dummy data
				ProcessLEDCommand(&ledcommand);
		        		
			
			}
		}
    }
    
    /***Taken from mCapTouch_timers.c ***/
    /* Check the interrupt Flag */
		if (PIR1bits.TMR2IF == HIGH)
		{
			if(channelSelect >= ScanChannelIndex)
			{
				/* Count upto MAX ADC channel though the actual scan happen only for channels which are enabled. 
				This helps to maintain the same interval for any channel when enabled or disabled. */
				if(channelSelect>= MAX_ADC_CHANNELS)
				{
					channelSelect = 0;	 
					dataReadyCTMU = 1;
				}
				else	
				{
					channelSelect++;
				}
			}
			else
			{
				/* Scans the CTMU channel for ADC voltage. It updates the "curRawData" and "actualValue" buffers. */
		   		mTouchCapPhy_ReadCTMU(ScanChannels[channelSelect]);

				/* Periodically average the channel data based on User configuration. */
				mTouchCapPhy_AverageData(ScanChannels[channelSelect]);		
				
				/* Set the channel number for scanning */
				channelSelect++;
			}
			
		     /* Clear the interrupt Flag */
		     PIR1bits.TMR2IF =LOW;
		}
    

}   

/*
High Priority interrupt routine is located in  CustomMRF24J40.c
*/
#pragma code lowVector=0x18
void LowVector(void){_asm goto LowISR _endasm}
#pragma code // Return to default code section



void main(void)
{
	BYTE i;


	InitializeHardware();
	
     MiApp_ProtocolInit(FALSE);
     MiApp_ConnectionMode(ENABLE_ALL_CONN);

	LCDInit();
    LCDClear(BLACK);

   PowerIndicator_SetDefault(&pwrIndicator);
   PowerIndicator_Update(&pwrIndicator);
       
   BarGraph_SetDefault(&redBar);
   BarGraph_SetDefault(&blueBar);
   BarGraph_SetDefault(&greenBar);    
   redBar.value = 120;
   redBar.left = 16;
   redBar.FillColor = RED;
   redBar.BorderColor = RED;
   redBar.BorderWeight = 2;
   redBar.OuterBorderWeight = 3;

   
   greenBar.value = 120;
   greenBar.left = 56;
   greenBar.FillColor = GREEN;
   greenBar.BorderColor = GREEN;
   greenBar.BorderWeight = 2;
   greenBar.OuterBorderWeight = 3;

           
   blueBar.value = 120;
   blueBar.left = 96;
   blueBar.FillColor = BLUE;
   blueBar.BorderColor = BLUE;
   blueBar.BorderWeight = 2;
   blueBar.OuterBorderWeight = 3;
        
        
	BarGraph_Draw(&redBar);
	BarGraph_Draw(&greenBar);
	BarGraph_Draw(&blueBar);
	
	Signal_SetDefault(&p2pSignal);
	Signal_Update(&p2pSignal);
	

	
	//SpkInit(); //not using speaker right now
	
	
	setupCapTouch();
	


	
	
	FindP2PConnection();
	
	Delay10KTCYx(10);
	
	RequestLEDTable();
	LEDTableRequestStamp = SystemTick;
 	
 	ActivityStamp = SystemTick; 

	/*Main multitasking loop */
	while(1)
	{
		/* The RF interrupt pin is shared with the power button */
		if (!PWR_BTN || (SystemTick - ActivityStamp) > IDLE_TIMEOUT)
		{
			if (!PowerButtonDownStamp)
			{
				PowerButtonDownStamp = SystemTick;
			}
			else
			{
				if ((SystemTick - PowerButtonDownStamp) > 2000 || (SystemTick - ActivityStamp) > IDLE_TIMEOUT)
				{
					if (PowerButtonDownStamp != 0)
					{
						
						//power button was held for over 2 seconds, turn off
	
						GoToSleep();
						
			
						
						
					
						
					}
				
				}	
			}	
		}
		else
		{
			PowerButtonDownStamp = 0;
		}		
			
		/*This portion of the loop checks on the capacitive touch keys */
		
		if(dataReadyCTMU == 1)			// This flag is set by Timer 2 ISR //when all channels have been read										
		{
			BYTE sliderLevel = 255;
			BYTE KeyDownCount = 0;
			DirectKey * ActiveKey;
			BYTE i;
			Set_ScanTimer_IF_Bit_State(DISABLE);              //Clear timer 2 SHORT flag
    		Set_ScanTimer_IE_Bit_State(DISABLE);              //Disable interrupt
    		Set_ScanTimer_ON_Bit_State(DISABLE) ;              //Stop timer 2
			dataReadyCTMU = 0;			//clear flag
			
			for (i=0; i < NUM_ELEM(KeyArray); i++)
			{
				if (KEY_PRESSED == mTouchCapAPI_GetStatusDirectButton(KeyArray[i]))
				{
					KeyDownCount++;
					ActiveKey = KeyArray[i];
					
				}
			}	
			if (MaintenenceMode)
			{
				if (KeyDownCount == 1) //ignore if simultaneous keys pressed
				{
					if (ActiveKey == &MaintenenceKey)
					{
						if ((SystemTick - ActiveKey->DebounceTick) > DEBOUNCE_MS)
		   		 		{
			   		 		ActiveKey->DebounceTick = SystemTick;
			   		 		ActivityStamp = SystemTick; 
							(*ActiveKey->touchfunction)();
		   				}
	   				}			
				}
			}
			else
			{	
				if (KeyDownCount == 1) //ignore if simultaneous keys pressed
				{
					if ((SystemTick - ActiveKey->DebounceTick) > DEBOUNCE_MS)
	   		 		{
		   		 		ActiveKey->DebounceTick = SystemTick;
		   		 		ActivityStamp = SystemTick; 
						(*ActiveKey->touchfunction)();
	   				}
				}
			}	
			
		
    		//check on slider
    		sliderLevel =  mTouchCapAPI_GetStatusSlider2Ch(&RGBSlider);
    		if(sliderLevel >= 0)
	    	{
		    	if (sliderLevel != RGBSlider.lastValue && sliderLevel != 255)
		    	{
			    	
			    	long barValue = 0;
			    	//float percent;

	        		
	        		/* //Diagnostic routine to print slider value
	        		char buffer[20];
	        		ultoa(sliderLevel, buffer);
	        		if (buffer[2] == 0)
	        		{
		        		buffer[3] = 0;
		        		buffer[2] = ' ';
		        	}	
		        	if (buffer[1] == 0)
		        	{
			        	buffer[3] = 0;
		        		buffer[2] = ' ';
		        		buffer[1] = ' ';
			        }	
	        		LCDString(WHITE, BLACK, buffer, 20,5);
	        		*/
	        	
	        	
	        		//this threats the slider level minimum as 10 and maximum as 75 and turns it into a number from 0 to 255
	        		if (sliderLevel < 10) sliderLevel = 10;
	        		sliderLevel -= 10;
	        		if (sliderLevel > 65) sliderLevel = 65;
	        		barValue = sliderLevel * 46500;
	        		barValue /= 10000;
	        		if (barValue > 255) barValue = 255;
	        		if (barValue < 0) barValue = 0;
	        		if (pwrIndicator.state)
	        		{
	        			RGBValueChange((BYTE) barValue);
	        		}
	        		ActivityStamp = SystemTick; 
	        	
	      		}  
	     	}
	     	
	     	Set_ScanTimer_IF_Bit_State(DISABLE);              //Clear flag
    		Set_ScanTimer_IE_Bit_State(ENABLE);              //Enable interrupt
    		Set_ScanTimer_ON_Bit_State(ENABLE) ;              //Run timer		
		}
		
		//Set the indicator to reflect the state of the connection if neccesary
		
		if (ConnectionTable[0].status.bits.isValid != p2pSignal.visible)
		{
			p2pSignal.visible = ConnectionTable[0].status.bits.isValid;
			Signal_Update(&p2pSignal);
		}
		
		if (FailCount > MAX_FAILURES)
		{
			//MiApp_RemoveConnection(ActivePANIndex);
		}	
		
		
		if (!ConnectionTable[0].status.bits.isValid)
		{
			//FindP2PConnection();
		}		

		/****************MIWI LOOP***************/		
		if( MiApp_MessageAvailable() )
        {
	        BYTE PayloadIndex;
	        BYTE ResponseCode = rxMessage.Payload[0];
	        
	        /*This is a message from the LED Controller reporting
	          the status of all LEDS */
	        if (ResponseCode == REQUEST_LED_TABLE_COMMAND)
	        {
		        pwrIndicator.state = rxMessage.Payload[1];
		        PowerIndicator_Update(&pwrIndicator);
		        LEDGroup.NumOfLEDs = rxMessage.Payload[2]; 
		        LEDGroup_SetDefault(&LEDGroup);
		        PayloadIndex = 2;
		    	for(i = 0; i < LEDGroup.NumOfLEDs; i++) 
			    { 
		       		LEDGroup.LEDs[i].address = rxMessage.Payload[++PayloadIndex];
		       		LEDGroup.LEDs[i].red = rxMessage.Payload[++PayloadIndex];
		       		LEDGroup.LEDs[i].green = rxMessage.Payload[++PayloadIndex];
		       		LEDGroup.LEDs[i].blue = rxMessage.Payload[++PayloadIndex];
		       		LEDGroup.LEDs[i].exists = 1;
		       		LEDGroup.LEDs[i].visible = 1;
		        }
		        LEDGroup_Update(&LEDGroup);
				if (LEDGroup.NumOfLEDs > 0)
				{
					redBar.value = LEDGroup.LEDs[0].red;
					greenBar.value = LEDGroup.LEDs[0].green;
					blueBar.value = LEDGroup.LEDs[0].blue;
					BarGraph_UpdateBar(&redBar);
					BarGraph_UpdateBar(&greenBar);
					BarGraph_UpdateBar(&blueBar);
				}
				GotLEDTable = 1;
 			}  
 			        
        	
        	
        	p2pSignal.LQQIv = rxMessage.PacketLQI;
        	p2pSignal.RSSIv = rxMessage.PacketRSSI;
        	Signal_Update(&p2pSignal);
        	
        	if (MaintenenceMode)
        	{
	        
	            char tmpBuffer[10];
	        	char MessageBuffer[20];	
	        	char length;
	 

	        	ultoa((unsigned long) rxMessage.PacketRSSI, tmpBuffer);
	        	strcpypgm2ram(MessageBuffer, "RSSI: ");
	        	strcat(MessageBuffer, tmpBuffer);
	        	length = strlen(MessageBuffer);
	        	MessageBuffer[length] = ' ';
	        	MessageBuffer[length+ 1] = ' ';
	        	MessageBuffer[length+ 2] = ' ';
	        	MessageBuffer[length+ 3] = 0;
	        	
	        	LCDString(WHITE, BLACK, MessageBuffer, 10,60);
	        	
	        	
	            ultoa((unsigned long) rxMessage.PacketLQI, tmpBuffer);
	        	strcpypgm2ram(MessageBuffer, "LQI: ");
	        	strcat(MessageBuffer, tmpBuffer);
	        	length = strlen(MessageBuffer);
	        	MessageBuffer[length] = ' ';
	        	MessageBuffer[length+ 1] = ' ';
	        	MessageBuffer[length+ 2] = ' ';
	        	MessageBuffer[length+ 3] = 0;
	        	LCDString(WHITE, BLACK, MessageBuffer, 10,75);
	        	
	        	if (ResponseCode == REQUEST_POWER_COMMAND)
	        	{
		        	char length;
		        	signed char i;
		        	int voltage = ((int)rxMessage.Payload[1]) << 8 | ((int)rxMessage.Payload[2]);
		        	int current = ((int)rxMessage.Payload[3]) << 8 | ((int)rxMessage.Payload[4]);
		        	ultoa((unsigned long) voltage, tmpBuffer);
		        	length = strlen(tmpBuffer);
		        	if (length < 4)
		        	{
			        	for (i=length-1; i >= 0; i--)
			        	{
				        	tmpBuffer[i+(4-length)] = tmpBuffer[i];
				        }
				        for (i=0;i<(4-length);i++)
				        {
					        tmpBuffer[i] = '0';
					    } 
			        }	
		        	tmpBuffer[5] = 0;
		        	tmpBuffer[4] = tmpBuffer[3];
		        	tmpBuffer[3] = tmpBuffer[2];
		        	tmpBuffer[2] = '.';
		        	
		        		
	        		strcpypgm2ram(MessageBuffer, "V: ");
	        		strcat(MessageBuffer, tmpBuffer);
	        		LCDString(WHITE, BLACK, MessageBuffer, 10,90);
	        		
	        		ultoa((unsigned long) current, tmpBuffer);
		        	length = strlen(tmpBuffer);
		        	
		        	if (length < 4)
		        	{
			        	for (i=(length-1); i >= 0; i--)
			        	{
				        	tmpBuffer[i+(4-length)] = tmpBuffer[i];
				        }
				        for (i=0;i<(4-length);i++)
				        {
					        tmpBuffer[i] = '0';
					    } 
			        }
			        
		        	tmpBuffer[5] = 0;
		        	tmpBuffer[4] = tmpBuffer[3];
		        	tmpBuffer[3] = tmpBuffer[2];
		        	tmpBuffer[2] = tmpBuffer[1];
		        	tmpBuffer[1] = '.';
		        	
		        		
	        		strcpypgm2ram(MessageBuffer, "I: ");
	        		strcat(MessageBuffer, tmpBuffer);
	        		LCDString(WHITE, BLACK, MessageBuffer, 10,105);
	        	
		        }	
	        	 
	        }	
        	
        	MiApp_DiscardMessage();
        } 

        else
        {
	        if (!GotLEDTable)
	        {
		        if ((SystemTick - LEDTableRequestStamp) > LED_TABLE_TIMEOUT)
		        {
			     	//MiApp_RemoveConnection(ActivePANIndex);
			     	RequestLEDTable();
			    } 
		        
		    } 		  	
		  	
	   		
        }
		
		
	}
	

}	



void InitializeHardware(void)
{
	/**Configure Oscillator for 64Mhz**/
	OSCCONbits.IRCF0 = 1;
	OSCCONbits.IRCF1 = 1;
	OSCCONbits.IRCF2 = 1;
	while(!OSCCONbits.HFIOFS);
	OSCTUNEbits.PLLEN = 1;
	OSCCONbits.SCS1 = 0;
	OSCCONbits.SCS0 = 0;
	
	/***Setup Pin states****/
	
	//start will all as inputs
	TRISA = 0xFF;
	TRISB = 0xFF;
	TRISC = 0xFF;
	
	//start with all digital
	ANSELA = 0;
	ANSELB = 0;
	ANSELC = 0;
	
	
	LCD_EN_TRIS = 0;
	BOOST_EN_TRIS = 0;
	
	LCD_EN = 1;
	BOOST_EN = 1;  
	
		PHY_CS_TRIS = 0;
      	PHY_CS = 1;
      	PHY_RESETn_TRIS = 0;
      	PHY_RESETn = 1;
      	
      	RF_INT_TRIS = 1; //Set RF Int as input
      	INTCON2bits.INTEDG0 = 0;
      	
        SDI_TRIS = 1;
        SDO_TRIS = 0;
        SCK_TRIS = 0;
        

        
        //Start with all pins in digital mode
	      ANSELA = 0;
	      ANSELB = 0;
	      ANSELC = 0;      
        
      
       
       
       
        
      //Enable pull up on our int on change pin
      
      INTCON2bits.RBPU = 0; //Enable pull ups globally
		WPUB = 0;
      
      //Set up power button
      PWR_BTN_TRIS = 1;
      PWR_WPUB = 1;
      

      
        
        
    /****Setup SPI ****/
     SPISSPCON = 0;
     SPISSPSTAT = 0x40;
     SPISSPCON = 1 | 0x20; //enable SPI, set clock speed
     
     
    /****Setup Interrupts***/
    
    //miwi interrupt
    RFEDGE = 0;
    RFIF = 0;
    RFIE = 1;
    
    INTCONbits.TMR0IE = 1; //Enable Timer 0 Interrupt
    
    //Set up Interrupt priorities
	//All low except Miwi
	
	IPR1 = 0;
	IPR2 = 0;
	IPR3 = 0;
	IPR4 = 0;
	IPR5 = 0;
	INTCON2bits.TMR0IP = 0;
	INTCON2bits.RBIP = 0;
	INTCON3bits.INT1IP = 0;
	INTCON3bits.INT2IP = 0;
	
	//RFIP = 1; //INT0 is always high priority
	
	RCONbits.IPEN = 1;		// Enable interrupt priorities
	
		// Globally Enable Both Interrupt Priorities
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;
    
    //turn off periphreals not in use.
    T5CONbits.TMR5ON = 0;
    PMD0bits.UART2MD = 1;
    PMD0bits.UART1MD = 1;
    PMD0bits.TMR6MD = 1;
    PMD0bits.TMR5MD = 1;
    PMD0bits.TMR4MD = 1;
    PMD0bits.TMR3MD = 1;
    PMD1bits.CCP5MD = 1;
    PMD1bits.CCP4MD = 1;
    PMD1bits.CCP3MD = 1;
    PMD1bits.CCP2MD = 1;
    PMD1bits.CCP1MD = 1;
    
    
    
}

rom const BYTE SELECTION_STATES[] = {0, 0b111, 0b100, 0b010, 0b001, 0xFF};
rom const BYTE * CurrentSelectionState = &SELECTION_STATES[1];
#define LAST_SELECTION_INDEX 4

void RGBValueChange(BYTE value)
{
	BYTE i;
		if (redBar.Selected)
   		{
	   		BYTE i;
	   		for (i=0; i < LEDGroup.NumOfLEDs;i++)
	   		{
		   		if (LEDGroup.LEDs[i].Selected)
		   		{
			   		LEDGroup.LEDs[i].red = value;
			   		setLEDState(i,SET_RED_COMMAND,value);
			   	}	
		   	}	
   			redBar.value = value;
   			BarGraph_UpdateBar(&redBar);
   		}
   		
   		if (greenBar.Selected)
   		{
	   		for (i=0; i < LEDGroup.NumOfLEDs;i++)
	   		{
		   		if (LEDGroup.LEDs[i].Selected)
		   		{
			   		LEDGroup.LEDs[i].green = value;
			   		setLEDState(i,SET_GREEN_COMMAND,value);
			   	}	
		   	}	
   			greenBar.value = value;
   			BarGraph_UpdateBar(&greenBar);
   		}
   		
   		if (blueBar.Selected)
   		{
	   		for (i=0; i < LEDGroup.NumOfLEDs;i++)
	   		{
		   		if (LEDGroup.LEDs[i].Selected)
		   		{
			   		LEDGroup.LEDs[i].blue = value;
			   		setLEDState(i,SET_BLUE_COMMAND,value);
			   	}	
		   	}	
   			blueBar.value = value;
   			BarGraph_UpdateBar(&blueBar);
   		}
}

void SpkInit(void)
{
	 /***configure Compare for speaker****/
       
       //Disbale SR Latch
		SRCON0 = 0;
		SRCON1 = 0;
       
       SPK_TRIS = 1;
       SPK_LAT = 0;
       
       //Turn on power to Timer 3
       PMD0bits.TMR3MD = 0;
       
       //Turon on power to CCP5
       PMD1bits.CCP5MD = 0;
       
        //Turon on power to CCP4
       PMD1bits.CCP4MD = 0;
       

       
       PIR3bits.TMR3GIF = 0;
       PIE3bits.TMR3GIE = 0;
       
       
       T3CONbits.TMR3ON =0;
       
       //Timer 3 uses FOSC/4
        T3CONbits.TMR3CS1 = 0;
        T3CONbits.TMR3CS0 = 0;
        
        T3CONbits.T3SOSCEN = 0; //no dedicated oscillator
        
        
        CCP5CON = 0;
        CCP4CON = 0;
        
        //1:1 Prescaler
        T3CONbits.T3CKPS1 = 0;
        T3CONbits.T3CKPS0 = 0;
       
       //CCP5: Toggle pin on compare match
       CCP5CONbits.CCP5M3= 0;
       CCP5CONbits.CCP5M2= 0;
       CCP5CONbits.CCP5M1= 1;
       CCP5CONbits.CCP5M0= 0;
       
       //CCP4: Clear timer on compare match
       CCP4CONbits.CCP4M3= 1;
       CCP4CONbits.CCP4M2= 0;
       CCP4CONbits.CCP4M1= 1;
       CCP4CONbits.CCP4M0= 1;
       
       
       //use timer 3 for CCP5
       CCPTMRS1bits.C5TSEL1 = 0;
       CCPTMRS1bits.C5TSEL0 = 1;
       
       //use timer 3 for CCP4
       CCPTMRS1bits.C4TSEL1 = 0;
       CCPTMRS1bits.C4TSEL0 = 1;
       
       CCPR5H = 10;
       CCPR5L = 127;
       
       CCPR4H = 10;
       CCPR4L = 130;
       
       T3CONbits.TMR3ON =0;
       
       
       
       
       //Disable output driver 
       SPK_TRIS = 1;
       
       
       SetSpkFreq(400);
       
       PIR4bits.CCP5IF = 0;
       PIE4bits.CCP5IE = 0;
       
       
}
void SetSpkFreq	(int freq)
{
	int period = ((CLOCK_FREQ / 4) / 2)	/ freq;
	CCPR5H = (period >> 8);
	CCPR4H = (period >> 8);
	CCPR5L = (period);
	CCPR4L = (period);
}	

void beep(void)
{
	SoundRequest sr;
	sr.duration = 30;
	sr.frequency = 1318;
	AddSoundRequest(&sr);
}

void longBeep(void) //indicate an error
{
	SoundRequest sr;
	sr.duration = 1000;
	sr.frequency = 400;
	AddSoundRequest(&sr);
}	
void SoundInto(void)
{
	SoundRequest sr;
	sr.duration = 94;
	sr.frequency = 262;
	AddSoundRequest(&sr);
	sr.duration = 16;
	sr.frequency = -1; //pause
	AddSoundRequest(&sr);
	sr.duration = 94;
	sr.frequency = 659;
	AddSoundRequest(&sr);
	sr.duration = 16;
	sr.frequency = -1;
	AddSoundRequest(&sr);
	sr.duration = 94;
	sr.frequency = 784;
	AddSoundRequest(&sr);
	sr.duration = 16;
	sr.frequency = -1;
	AddSoundRequest(&sr);
	sr.duration = 250;
	sr.frequency = 1046;
	AddSoundRequest(&sr);

}	

void AddSoundRequest(SoundRequest * req)
{
	req->WasServiced = 0;
	*SoundRequestPtr = *req;
	IncrementRequestPtr();
}	


void FindP2PConnection(void)
{
	BYTE i;
    BYTE NumberOfPANs;
    BYTE IndexOfTargetPAN;
 
    char strConnecting[] = "Connecting";
    

	    drawFilledBox(BLACK, 0,20, 128, 48);
	    LCDString(WHITE, BLACK, strConnecting, 14,30);
	    RotatingDots = 1;
	    ScanStartStamp = SystemTick;
		//Look for an open P2P connection
	    do {
		     NumberOfPANs = MiApp_SearchConnection(6, 0xFFFFFFFF);
		     
		     if ((SystemTick-ScanStartStamp) > 60000)
		     {
			     GoToSleep();
			     
			 } 
	     
	     } while (NumberOfPANs < 1);
	     
	     for (i=0;i<NumberOfPANs;i++)
	     {
	     	 if(ActiveScanResults[i].PANID.Val == 0x12A0)
	     	 {
	      		 IndexOfTargetPAN = i;
	   		  } 
	    }  
	    //Connect to the P2P connection
	     ScanStartStamp = SystemTick;
	     ActivePANIndex = 0xFF;
	    while( (ActivePANIndex = MiApp_EstablishConnection(IndexOfTargetPAN, CONN_MODE_DIRECT)) == 0xFF )
	    {
		   if ((SystemTick-ScanStartStamp) > 45000)
	     	{ 
		     	GoToSleep();
			     
		    }	
		}

    
    RotatingDots = 0;
    drawFilledBox(BLACK, 0,20, 128, 48);
    //SoundInto();
}	


void SoundOn(void)
{
	SPK_TRIS = 0;
	SoundStartTick = SystemTick;
	T3CONbits.TMR3ON = 1;	
}	
void SoundOff(void)
{
	T3CONbits.TMR3ON = 0;
	SPK_TRIS = 1;
	SoundPause = 0;
}	
void IncrementRequestPtr(void)
{
	if ((SoundRequestPtr - &SoundQue[0]) < (SOUND_QUE_LENGTH-1))
	{
		SoundRequestPtr++;	
	}
	else
	{
		SoundRequestPtr = &SoundQue[0];
	}		 
	
}	
void IncrementServicePtr(void)
{
	if ((SoundServicePtr - &SoundQue[0]) < (SOUND_QUE_LENGTH-1))
	{
		
		SoundServicePtr++;	
	}
	else
	{
		SoundServicePtr = &SoundQue[0];
	}	
}	
void RequestLEDTable(void)
{
	BYTE SendSuccess;
	MiApp_FlushTx();
	MiApp_WriteData(REQUEST_LED_TABLE_COMMAND);
	
	//verification bytes
	MiApp_WriteData(110);
	MiApp_WriteData(210);
	
	SendSuccess=MiApp_UnicastConnection(ActivePANIndex, FALSE);
	Nop();
	Nop();
	Nop();
}
void mTouchCapApp_PortSetup(void)
{
ANSELA = 1;
REDKEY_TRIS = 1;
GREENKEY_TRIS = 1;
BLUEKEY_TRIS = 1;
POWERKEY_TRIS = 1;
SELECTLEDKEY_TRIS = 1;
SELECTALLKEY_TRIS = 1;
RESETRGBKEY_TRIS = 1;
MAINTENENCEKEY_TRIS = 1;
SLIDER_TRIS1 = 1;
SLIDER_TRIS2 = 1;

ADPCF_REDKEY(1)
ADPCF_GREENKEY(1)
ADPCF_BLUEKEY(1)
ADPCF_POWERKEY(1)
ADPCF_SELECTLEDKEY(1)
ADPCF_SELECTALLKEY(1)
ADPCF_RESETRGBKEY(1)
ADPCF_MAINTENENCEKEY(1)
ADPCF_SLIDER(1)

}


	
void setupCapTouch(void)
{

	mTouchCapAPI_SetUpChannelDirectKey(&RedKey,REDKEY_CHANNEL,DEFAULT_TRIP_VALUE,DECODE_METHOD_PRESS_AND_RELEASE,FILTER_METHOD_SLOWAVERAGE);
	RedKey.DebounceTick = 0;
	RedKey.touchfunction = &RedKeyTouchEvent;
	mTouchCapAPI_SetUpChannelDirectKey(&GreenKey,GREENKEY_CHANNEL,DEFAULT_TRIP_VALUE,DECODE_METHOD_PRESS_AND_RELEASE,FILTER_METHOD_SLOWAVERAGE);
	GreenKey.DebounceTick = 0;
	GreenKey.touchfunction = &GreenKeyTouchEvent;
	mTouchCapAPI_SetUpChannelDirectKey(&BlueKey,BLUEKEY_CHANNEL ,DEFAULT_TRIP_VALUE,DECODE_METHOD_PRESS_AND_RELEASE,FILTER_METHOD_SLOWAVERAGE);
	BlueKey.DebounceTick = 0;
	BlueKey.touchfunction = &BlueKeyTouchEvent;
	mTouchCapAPI_SetUpChannelDirectKey(&SelectLEDKey,SELECTLEDKEY_CHANNEL,DEFAULT_TRIP_VALUE,DECODE_METHOD_PRESS_AND_RELEASE,FILTER_METHOD_SLOWAVERAGE);
	SelectLEDKey.DebounceTick = 0;
	SelectLEDKey.touchfunction = &SelectLEDKeyTouchEvent;
	mTouchCapAPI_SetUpChannelDirectKey(&SelectAllKey,SELECTALLKEY_CHANNEL,DEFAULT_TRIP_VALUE,DECODE_METHOD_PRESS_AND_RELEASE,FILTER_METHOD_SLOWAVERAGE);
	SelectAllKey.DebounceTick = 0;			
	SelectAllKey.touchfunction = &SelectAllKeyTouchEvent;	
	mTouchCapAPI_SetUpChannelDirectKey(&ResetRGBKey,RESETRGBKEY_CHANNEL,DEFAULT_TRIP_VALUE,DECODE_METHOD_PRESS_AND_RELEASE,FILTER_METHOD_SLOWAVERAGE);
	ResetRGBKey.DebounceTick = 0;
	ResetRGBKey.touchfunction = &ResetRGBKeyTouchEvent;
	mTouchCapAPI_SetUpChannelDirectKey(&MaintenenceKey,MAINTENENCEKEY_CHANNEL,DEFAULT_TRIP_VALUE,DECODE_METHOD_PRESS_AND_RELEASE,FILTER_METHOD_SLOWAVERAGE);
	MaintenenceKey.DebounceTick = 0;	
	MaintenenceKey.touchfunction = &MaintenenceKeyTouchEvent;
	mTouchCapAPI_SetUpChannelDirectKey(&PowerKey,POWERKEY_CHANNEL,DEFAULT_TRIP_VALUE,DECODE_METHOD_PRESS_AND_RELEASE,FILTER_METHOD_SLOWAVERAGE);
	PowerKey.DebounceTick = 0;
	PowerKey.touchfunction = &PowerKeyTouchEvent;
	
	mTouchCapAPI_SetUpChannelSlider2Ch(&RGBSlider, SLIDER_CHANNEL1, SLIDER_CHANNEL2, 100 ,DECODE_METHOD_PRESS_ASSERT,FILTER_METHOD_GATEDAVERAGE);
	
	mTouchCap_Init();

}	


/*This function came from the Microchip 2CH slider demo but has been modified to
	avoid usage of floats */


WORD mTouchCapApp_TwoChannelSliderLogic(Slider2Ch *Slider)
{
	WORD Channel_sum;
	/*
	float Percent_Level = 0; 
	float Percent_R = 0;
	float Percent_L = 0;
	*/
	int Percent_Level = 0; 
	int Percent_R = 0;
	int Percent_L = 0;
	WORD Channel_L;
	WORD Channel_R;


	Channel_L = averageData[Slider->Channel_Num[0]] - curRawData[Slider->Channel_Num[0]];
	Channel_R = averageData[Slider->Channel_Num[1]] - curRawData[Slider->Channel_Num[1]];

	Channel_sum = (Channel_L + Channel_R);
	
	if(Channel_sum <= 0)		// to avoid divide by zero factor
	{
			Percent_L =0;
			Percent_R =0;
			Percent_Level =0;
	}
	else 
	{
			/* To check the Right Sensor values( channel 1) */
			if (Channel_R == 0)  Percent_R = 0;
			else
			{	
				/*	
				Percent_R = ((float)Channel_R)/(Channel_sum);				
				Percent_R = 100 * Percent_R;
				*/
				Percent_R = (100 * Channel_R)/(Channel_sum);
			
				
			}

			/* To check the Left Sensor values( channel 0) */
			if (Channel_L == 0)  Percent_L = 0; 
			else
			{
				/*
				Percent_L = ((float)Channel_L/Channel_sum);				
				Percent_L = 100* Percent_L;
				*/
				Percent_L = (100 * Channel_L) / Channel_sum;
			}
						
			if ((Percent_R >= 0) && (Percent_R <= TOLERANCE_LIMIT_MAX_UNTOUCH) ) // Tolerance for interference in RIGHT channel while measuring LEFT
			{
				if(Percent_L <= MAX_LEFT_CHANNEL_SCALE_OF_SLIDER)	Percent_Level = (  MAX_LEFT_CHANNEL_SCALE_OF_SLIDER - Percent_L); // sliding from MAX area of LEFT channel towards mid	
				else												Percent_Level = (  100 - Percent_L);
			}
			else if( (Percent_L >= 0) && (Percent_L <= TOLERANCE_LIMIT_MAX_UNTOUCH) ) // Tolerance for interference in LEFT channel while measuring RIGHT
			{
				if(Percent_R >= MAX_LEFT_CHANNEL_SCALE_OF_SLIDER)
				{
					Percent_Level = (  Percent_R - MAX_LEFT_CHANNEL_SCALE_OF_SLIDER); // sliding from MAX area of LEFT channel towards mid	
					Percent_Level += MAX_LEFT_CHANNEL_SCALE_OF_SLIDER;
				}
				else												Percent_Level = Percent_R;
			}
			else if(Percent_L !=0 && Percent_R !=0) 
			{	
				if(Percent_L < Percent_R)
					Percent_Level = Percent_R ;
				else if (Percent_L > Percent_R)	
					Percent_Level = ( 100 - Percent_L);
				else if (Percent_L == Percent_R)
					Percent_Level = (Percent_R + (MAX_LEFT_CHANNEL_SCALE_OF_SLIDER - Percent_L))/2 ;								

				if( (Percent_Level >= MAX_LEFT_CHANNEL_SCALE_OF_SLIDER) )		Percent_Level = Percent_Level - (100-MAX_LEFT_CHANNEL_SCALE_OF_SLIDER);	
				if( (Percent_Level <= MIN_RIGHT_CHANNEL_SCALE_OF_SLIDER) )	Percent_Level = Percent_Level + MIN_RIGHT_CHANNEL_SCALE_OF_SLIDER;	

			}	
	}

	return Percent_Level;
}



void RedKeyTouchEvent()
{
	redBar.Selected = !redBar.Selected;
	BarGraph_UpdateBar(&redBar);
}	
void GreenKeyTouchEvent()
{
	greenBar.Selected = !greenBar.Selected;   
	BarGraph_UpdateBar(&greenBar);   
}	
void BlueKeyTouchEvent()
{
	blueBar.Selected = !blueBar.Selected;  
	BarGraph_UpdateBar(&blueBar);
}	
void SelectLEDKeyTouchEvent()
{
	if (LEDGroup.NumOfLEDs > 1)
	{
		if (LEDGroup.LEDs[0].Selected && LEDGroup.LEDs[1].Selected) 
		//If two are selected they all are selected so change to only the
		//first led selected
		{
			LEDGroup_SelectAll(&LEDGroup,0);
			LEDGroup.LEDs[0].Selected = 1;
			LED_Update(&(LEDGroup.LEDs[0]));
			redBar.value = LEDGroup.LEDs[0].red;
   			BarGraph_UpdateBar(&redBar);
   			greenBar.value = LEDGroup.LEDs[0].green;
   			BarGraph_UpdateBar(&greenBar);
   			blueBar.value = LEDGroup.LEDs[0].blue;
   			BarGraph_UpdateBar(&blueBar);
		}
				else //we are in single selection mode
		{
			BYTE i;
			for (i=0; i < LEDGroup.NumOfLEDs; i++)
			{
				if (LEDGroup.LEDs[i].Selected)
				{
					if (i < (LEDGroup.NumOfLEDs - 1))
					{
	
		
						LEDGroup.LEDs[i].Selected = 0;
						LEDGroup.LEDs[i + 1].Selected = 1;
						LED_Update(&(LEDGroup.LEDs[i]));
						LED_Update(&(LEDGroup.LEDs[i + 1]));
						redBar.value = LEDGroup.LEDs[i + 1].red;
   						BarGraph_UpdateBar(&redBar);
   						greenBar.value = LEDGroup.LEDs[i + 1].green;
   						BarGraph_UpdateBar(&greenBar);
   						blueBar.value = LEDGroup.LEDs[i + 1].blue;
   						BarGraph_UpdateBar(&blueBar);
					}
					else
					{
						LEDGroup.LEDs[0].Selected = 1;
						LEDGroup.LEDs[i].Selected = 0;
						LED_Update(&(LEDGroup.LEDs[i]));
						LED_Update(&(LEDGroup.LEDs[0]));
						redBar.value = LEDGroup.LEDs[0].red;
   						BarGraph_UpdateBar(&redBar);
   						greenBar.value = LEDGroup.LEDs[0].green;
   						BarGraph_UpdateBar(&greenBar);
   						blueBar.value = LEDGroup.LEDs[0].blue;
   						BarGraph_UpdateBar(&blueBar);
					}
					break;		
				}	
			}	
		}	
			
	}	

}	
void SelectAllKeyTouchEvent()
{
	LEDGroup_SelectAll(&LEDGroup,1);
	if (LEDGroup.NumOfLEDs > 0)
	{
		LED_Update(&(LEDGroup.LEDs[0]));
		redBar.value = LEDGroup.LEDs[0].red;
	   	BarGraph_UpdateBar(&redBar);
	   	greenBar.value = LEDGroup.LEDs[0].green;
	   	BarGraph_UpdateBar(&greenBar);
	   	blueBar.value = LEDGroup.LEDs[0].blue;
	   	BarGraph_UpdateBar(&blueBar);
 	}  	
}	
void ResetRGBKeyTouchEvent()
{
	BYTE topValue = redBar.value;
	ResetRGBKey.DebounceTick = SystemTick;
   	if (greenBar.value > topValue) topValue = greenBar.value;
   	if (blueBar.value > topValue) topValue = blueBar.value;
   	redBar.value = topValue;
   	greenBar.value = topValue;
   	blueBar.value = topValue;
   	BarGraph_UpdateBar(&redBar);
   	BarGraph_UpdateBar(&greenBar); 
   	BarGraph_UpdateBar(&blueBar);
}	
void MaintenenceKeyTouchEvent()
{
	MaintenenceMode = !MaintenenceMode;
	if (MaintenenceMode)
	{
		drawFilledBox(BLACK, 0, 56, 130, 130);
	}
	else 
	{
		drawFilledBox(BLACK, 0, 56, 130, 130);
		BarGraph_Draw(&redBar);
		BarGraph_Draw(&greenBar);
		BarGraph_Draw(&blueBar);
	}		
}	
void PowerKeyTouchEvent()
{
	LEDCommand ledcommand;
	pwrIndicator.state = !pwrIndicator.state;
	ledcommand.commandcode = POWER_COMMAND;
	ledcommand.param1 = pwrIndicator.state;
	ledcommand.param2 = 11; //verification byte
	ProcessLEDCommand(&ledcommand);
	PowerIndicator_Update(&pwrIndicator);
}

void setLEDState(BYTE LEDIndex, BYTE ColorCode, BYTE val)
{
     BYTE success;
	 MiApp_FlushTx();
	 MiApp_WriteData(ColorCode);
     MiApp_WriteData(LEDGroup.LEDs[LEDIndex].address);
     MiApp_WriteData(val);
     success = MiApp_UnicastConnection(ActivePANIndex, FALSE);
     if (success)
     {
	     FailCount = 0;
	 } 
	 else
	 {
		 FailCount++;
	 } 
}	

void ProcessLEDCommand(LEDCommand * ledcommand)
{
	  BYTE success;
	  MiApp_FlushTx();	   
      MiApp_WriteData(ledcommand->commandcode);
      MiApp_WriteData(ledcommand->param1);
      MiApp_WriteData(ledcommand->param2);
      success = MiApp_UnicastConnection(ActivePANIndex, FALSE);
     if (success)
     {
	     FailCount = 0;
	 } 
	 else
	 {
		 FailCount++;
	 } 
}

void GoToSleep(void)
{
	long debounceCount;
	//put RF card to sleep
	
	MiApp_TransceiverPowerState(POWER_STATE_SLEEP);
	
	
	//change all to inputs
	TRISA = 0xFF;
	TRISB = 0xFF;
	TRISC = 0xFF;
	

	
	//power control pins
	LCD_EN_TRIS = 0;
	BOOST_EN_TRIS = 0;
	
	//power off to boost and lcd	
	LCD_EN = 0;
	BOOST_EN = 0; 
	
	//LCD lines high impedance
	
	 LCD_RES_TRIS = 1; // TRISCbits.TRISC0
	 LCD_CS_TRIS = 1; // TRISCbits.TRISC1
	 LCD_DIO_TRIS = 1; // TRISAbits.TRISA6
	 LCD_SCK_TRIS = 1; // TRISAbits.TRISA7

	
	
	//All captouch keeps digital and grounded
	ANSELA = 0;
	ANSELB = 0;
	ANSELC = 0;
	
	 REDKEY_TRIS =  0; //		TRISAbits.TRISA0
 GREENKEY_TRIS =  0; //  		TRISAbits.TRISA1
 BLUEKEY_TRIS =  0; // 		TRISAbits.TRISA5
 POWERKEY_TRIS =  0; //  		TRISBbits.TRISB5
 SELECTLEDKEY_TRIS =  0; // 		TRISCbits.TRISC2
 SELECTALLKEY_TRIS =  0; //  		TRISCbits.TRISC3
 RESETRGBKEY_TRIS =  0; // 		TRISCbits.TRISC4
 MAINTENENCEKEY_TRIS =  0; //  		TRISCbits.TRISC5
 
  SLIDER_TRIS1 =  0; //	TRISCbits.TRISC6
 SLIDER_TRIS2 =  0; //	TRISCbits.TRISC7

 REDKEY_LAT =  0; //		LATAbits.LATA0
 GREENKEY_LAT =  0; //  		LATAbits.LATA1
 BLUEKEY_LAT =  0; // 		LATAbits.LATA5
 POWERKEY_LAT =  0; //  		LATBbits.LATB5
 SELECTLEDKEY_LAT =  0; // 		LATCbits.LATC2
 SELECTALLKEY_LAT =  0; //  		LATCbits.LATC3
 RESETRGBKEY_LAT =  0; // 		LATCbits.LATC4
 MAINTENENCEKEY_LAT =  0; //  		LATCbits.LATC5


 SLIDER_LAT1 =  0; //	LATCbits.LATC6
 SLIDER_LAT2 =  0; //	LATCbits.LATC7
 //turn off ADC
 ADCON0bits.ADON = 0;
 CTMUCONHbits.CTMUEN = 0;
 
 //turn off SPI
 SSP2CON1bits.SSPEN = 0;
 
 SDI_TRIS = 1;
 SDO_TRIS = 1;
 SCK_TRIS = 1;
 
 
 
 //disable all periphreal modules
 PMD0 = 0xFF;
 PMD1 = 0xFF;
 PMD2 = 0xFF;

	//debounce
	debounceCount = SystemTick;
	while((SystemTick - debounceCount) < 1000);

//enable interrupt on change
PWR_IOCB = 1;
INTCONbits.RBIE = 1;

//disable interrupts

INTCONbits.GIE = 0;


Nop();
Nop();					



Sleep();
//These are the instructions that will execute upon wake-up

while (PWR_BTN)
{
	INTCONbits.RBIF = 0;
	Sleep();
}	

Nop();
Nop();
Reset(); //reset after wake-up



	
}			